@@ -33,10 +33,17 @@ def package(package_type, letter, package_name, package_file):
                              example: Django-1.5.0.tar.gz
     '''
     egg_filename = join(get_base_path(), package_name, package_file)
-    url = app.config['PYPI_URL'] + 'packages/%s/%s/%s/%s' % (package_type,
-                                                             letter,
-                                                             package_name,
-                                                             package_file)
+    
+    remote = request.args.get('remote')
+    
+    if remote:
+        # the requested link is not on pypi.python.org, we need to use the remote URL
+        url = remote
+    else:
+        url = app.config['PYPI_URL'] + 'packages/%s/%s/%s/%s' % (package_type,
+                                                                 letter,
+                                                                 package_name,
+                                                                 package_file)
     if request.method == 'HEAD':
         # in this case the content type of the file is what is
         # required
@@ -63,9 +70,6 @@ def package(package_type, letter, package_name, package_file):
         # Downloads the egg from pypi and saves it locally, then
         # it will return it.
 
-        # TODO: there are some problem here with some packages. For
-        #       example: py-bcrypt
-
         package_path = get_package_path(package_name)
         app.logger.debug('Starting to download: %s using the url: %s',
                          package_file, url)
